==> xcomponent-0.1.4.tar.gz: xcomponent-0.1.4/src/rust/expression/tokens.rs <==
use std::{fmt, str::FromStr};

use pyo3::prelude::*;

use crate::markup::tokens::XNode;

#[pyclass(eq, eq_int)]
#[derive(Debug, Clone, PartialEq)]
pub enum ExpType {
    Expression,
    Ident,
    Operator,
    String,
    Integer,
    Boolean,
}

#[derive(Debug, PartialEq, Eq)]
pub struct OperatorErr;

#[derive(Debug, Clone, PartialEq, Eq)]
pub enum Operator {
    Add,
    Sub,
    Mul,
    Div,
    And,
    Or,
    Eq,
    Neq,
    Gt,
    Gte,
    Lt,
    Lte,
}

impl FromStr for Operator {
    type Err = OperatorErr;

    fn from_str(op: &str) -> Result<Self, Self::Err> {
        match op {
            "+" => Ok(Operator::Add),
            "-" => Ok(Operator::Sub),
            "*" => Ok(Operator::Mul),
            "/" => Ok(Operator::Div),
            "and" => Ok(Operator::And),
            "or" => Ok(Operator::Or),
            "==" => Ok(Operator::Eq),
            "!=" => Ok(Operator::Neq),
            ">" => Ok(Operator::Gt),
            "<" => Ok(Operator::Lt),
            ">=" => Ok(Operator::Gte),
            "<=" => Ok(Operator::Lte),
            _ => Err(OperatorErr),
        }
    }
}

impl fmt::Display for Operator {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        let op = match self {
            Operator::Add => "+",
            Operator::Sub => "-",
            Operator::Mul => "*",
            Operator::Div => "/",
            Operator::And => "and",
            Operator::Or => "or",
            Operator::Eq => "==",
            Operator::Neq => "!=",
            Operator::Gt => ">",
            Operator::Lt => "<",
            Operator::Gte => ">=",
            Operator::Lte => "<=",
        };
        write!(f, "{}", op)
    }
}

#[derive(Debug, Clone, PartialEq)]
pub struct FunctionCall {
    ident: String,
    params: Vec<ExpressionToken>,
}

impl FunctionCall {
    pub fn new(ident: String, params: Vec<ExpressionToken>) -> Self {
        FunctionCall { ident, params }
    }
    pub fn ident(&self) -> &str {
        return self.ident.as_str();
    }
    pub fn params(&self) -> &Vec<ExpressionToken> {
        return self.params.as_ref();
    }
}

impl fmt::Display for FunctionCall {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        let params = (self
            .params
            .iter()
            .map(|p| format!("{}", p))
            .collect::<Vec<_>>())
        .join(",");
        write!(f, "{}({})", self.ident, params)
    }
}

#[derive(Debug, Clone, PartialEq)]
pub enum ExpressionToken {
    BinaryExpression(Vec<ExpressionToken>),
    Ident(String),
    Operator(Operator),
    String(String),
    Integer(isize),
    Boolean(bool),
    XNode(XNode),
    FuncCall(FunctionCall),
    IfExpression {
        condition: Box<ExpressionToken>,
        then_branch: Box<ExpressionToken>,
        else_branch: Option<Box<ExpressionToken>>,
    },
    ForExpression {
        ident: String,
        iterable: Box<ExpressionToken>,
        body: Box<ExpressionToken>,
    },
}

impl std::fmt::Display for ExpressionToken {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            ExpressionToken::BinaryExpression(children) => {
                write!(
                    f,
                    "{}",
                    children.iter().map(|v| v.to_string()).collect::<String>()
                )
            }
            ExpressionToken::Ident(ident) => {
                write!(f, "{}", ident)
            }
            ExpressionToken::Operator(op) => write!(f, " {} ", op.to_string()),
            ExpressionToken::String(value) => {
                write!(f, "\"{}\"", value.replace('"', "\\\""))
            }
            ExpressionToken::Integer(value) => write!(f, "{}", value),
            ExpressionToken::Boolean(value) => write!(f, "{}", value),
            ExpressionToken::XNode(n) => write!(f, "{}", n),
            ExpressionToken::FuncCall(func) => write!(f, "{}", func),
            ExpressionToken::IfExpression {
                condition,
                then_branch,
                else_branch,
            } => match else_branch {
                None => write!(f, "if {} {{ {} }}", condition, then_branch),
                Some(else_branch) => {
                    write!(
                        f,
                        "if {} {{ {} }} else {{ {} }}",
                        condition, then_branch, else_branch
                    )
                }
            },
            ExpressionToken::ForExpression {
                ident,
                iterable,
                body,
            } => write!(f, "for {} in {} {{ {} }}", ident, iterable, body),
        }
    }
}

==> xcomponent-0.2.0.tar.gz: xcomponent-0.2.0/src/rust/expression/tokens.rs <==
use std::{collections::HashMap, fmt, str::FromStr};

use crate::markup::tokens::XNode;
use pyo3::prelude::*;

#[pyclass(eq, eq_int)]
#[derive(Debug, Clone, PartialEq)]
pub enum ExpType {
    Expression,
    Ident,
    Operator,
    String,
    Integer,
    Boolean,
}

#[derive(Debug, PartialEq, Eq)]
pub struct OperatorErr;

#[derive(Debug, Clone, PartialEq, Eq)]
pub enum Operator {
    Add,
    Sub,
    Mul,
    Div,
    And,
    Or,
    Eq,
    Neq,
    Gt,
    Gte,
    Lt,
    Lte,
}

impl FromStr for Operator {
    type Err = OperatorErr;

    fn from_str(op: &str) -> Result<Self, Self::Err> {
        match op {
            "+" => Ok(Operator::Add),
            "-" => Ok(Operator::Sub),
            "*" => Ok(Operator::Mul),
            "/" => Ok(Operator::Div),
            "and" => Ok(Operator::And),
            "or" => Ok(Operator::Or),
            "==" => Ok(Operator::Eq),
            "!=" => Ok(Operator::Neq),
            ">" => Ok(Operator::Gt),
            "<" => Ok(Operator::Lt),
            ">=" => Ok(Operator::Gte),
            "<=" => Ok(Operator::Lte),
            _ => Err(OperatorErr),
        }
    }
}

impl fmt::Display for Operator {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        let op = match self {
            Operator::Add => "+",
            Operator::Sub => "-",
            Operator::Mul => "*",
            Operator::Div => "/",
            Operator::And => "and",
            Operator::Or => "or",
            Operator::Eq => "==",
            Operator::Neq => "!=",
            Operator::Gt => ">",
            Operator::Lt => "<",
            Operator::Gte => ">=",
            Operator::Lte => "<=",
        };
        write!(f, "{}", op)
    }
}

#[derive(Debug, Clone, PartialEq)]
pub enum PostfixOp {
    Field(String),
    Index(Box<ExpressionToken>),
    Call {
        args: Vec<ExpressionToken>,
        kwargs: HashMap<String, ExpressionToken>,
    },
}

#[derive(Debug, Clone, PartialEq)]
pub enum ExpressionToken {
    BinaryExpression(Vec<ExpressionToken>),
    Ident(String),
    Operator(Operator),
    String(String),
    // Uuid(String),
    Integer(isize),
    Boolean(bool),
    XNode(XNode),
    PostfixOp(PostfixOp),
    IfExpression {
        condition: Box<ExpressionToken>,
        then_branch: Box<ExpressionToken>,
        else_branch: Option<Box<ExpressionToken>>,
    },
    ForExpression {
        ident: String,
        iterable: Box<ExpressionToken>,
        body: Box<ExpressionToken>,
    },
    Noop,
}

impl std::fmt::Display for ExpressionToken {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            ExpressionToken::BinaryExpression(children) => {
                write!(
                    f,
                    "{}",
                    children.iter().map(|v| v.to_string()).collect::<String>()
                )
            }
            ExpressionToken::Ident(ident) => {
                write!(f, "{}", ident)
            }
            ExpressionToken::Operator(op) => write!(f, " {} ", op.to_string()),
            ExpressionToken::String(value) => {
                write!(f, "\"{}\"", value.replace('"', "\\\""))
            }
            // ExpressionToken::Uuid(value) => write!(f, "\"{}\"", value),
            ExpressionToken::Integer(value) => write!(f, "{}", value),
            ExpressionToken::Boolean(value) => write!(f, "{}", value),
            ExpressionToken::XNode(n) => write!(f, "{}", n),
            ExpressionToken::PostfixOp(op) => match op {
                PostfixOp::Field(field) => write!(f, ".{}", field),
                PostfixOp::Index(index) => write!(f, "[{}]", index),
                // FIXME, display the args and kwargs properly
                PostfixOp::Call { args, kwargs } => write!(f, "({:?}, {:?})", args, kwargs),
            },
            ExpressionToken::IfExpression {
                condition,
                then_branch,
                else_branch,
            } => match else_branch {
                None => write!(f, "if {} {{ {} }}", condition, then_branch),
                Some(else_branch) => {
                    write!(
                        f,
                        "if {} {{ {} }} else {{ {} }}",
                        condition, then_branch, else_branch
                    )
                }
            },
            ExpressionToken::ForExpression {
                ident,
                iterable,
                body,
            } => write!(f, "for {} in {} {{ {} }}", ident, iterable, body),
            ExpressionToken::Noop => write!(f, ""), // ??
        }
    }
}
